#
# arps.pl - Display the ntop's ARP Cache.
#
# Copyright (C) 2000-2003 Rocco Carbone <rocco@ntop.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

# what I need is
use lib qw(plugins/pep);
require "fmthtml.pl";
require "sorting.pl";

#
# useful variables
#
$__progname__   = "arps.pl";
$__version__    = "0.0.3";
$__updated__    = "2003 Mar 06";
$__boottime__   = time();
$__bootdate__   = localtime($__boottime__);

# Hey Netscape, this is for you
my $htmltitle   = "ntop ARP Cache";
my $bodyname    = "ARP Cache Report";
my $title       = "ARP Cache";
my $description = "Shows ntop's ARP Cache";

my $infobg      = '"#cccccc"';
my $headerbg    = '"#ccccff"';
my @bgcolors    = ('"#ffffff"', '"#eeeeee"');
my $j = 0;

my $width = '50%';

@reporttypes = ("ipaddress") unless @reporttypes;
@sorttypes = ("ascending") unless @sorttypes;

#
# get the names of all the perl hashes built in the PeP plugin,
# maintained in the %pephosts hash and now available in this script
#
my @pepnames = values %pephosts;

#
# sort hw address alphabetically and ip address numerically
#
my @sorted_by_hostname      = sort { $a->{hostname}  cmp $b->{hostname} }  @pepnames;
my @sorted_by_ipaddress     = sort ipnumerically @pepnames;
my @sorted_by_hwaddress     = sort { $a->{hwaddress} cmp $b->{hwaddress} } @pepnames;

my @sorted_by_rev_hostname  = reverse @sorted_by_hostname;
my @sorted_by_rev_ipaddress = reverse @sorted_by_ipaddress;
my @sorted_by_rev_hwaddress = reverse @sorted_by_hwaddress;

#
# other useful variables
#
my $items_per_section = 25;
my $total_items       = @pepnames;                                 # items to display
my $sections          = int($total_items / $items_per_section);    # sections to create

if ($total_items % $items_per_section) {
  $sections++;
}

select STDOUT;

#
# print HTML header and title
#
html_header($htmltitle);

#
# start printing HTML body
#
html_start_body($bodyname);

#
# do the job
#
display_cache();

#
# finish HTML output
#
html_stop_body();


#
# read the information from the hash and display them in tables
# accordingly to the given specification
#
sub display_cache {
  #
  # print report's title with its brief description
  #
  print "<center><font size=6><b>$description</b></font></center>\n";

  foreach my $reporttype (@reporttypes) {

    my @sorted     = eval("\@sorted_by_$reporttype");
    my @sorted_rev = eval("\@sorted_by_rev_$reporttype");

    foreach my $sorttype (@sorttypes) {
      my $current = 0;

      for ($section = 1; $section <= $sections; $section++) {
	# the sub array for this section
	my @sub_sorted;

	#
	# useful calculations before starting playing with sections
	#
	my $last = $current + $items_per_section - 1;

	if ($last >= $total_items) {
	  $last = $total_items - 1;
	}
	my $itemno = $last - $current + 1;

	#
	# isolate the sub array containing only the elements candidates for display
	#
	if ($sorttype eq "ascending") {
	  @sub_sorted = @sorted[$current..$last];
	}
	else {
	  @sub_sorted = @sorted_rev[$current..$last];
	}

	#
	# display the table description and its header
	#
	display_table_description($itemno, $section, $current, $last, $sortype, $reporttype);
	display_table_header($itemno, $sortype);
	display_table(\@sub_sorted);
	display_table_footer();

	$current += $items_per_section;
      }   # next section
    }     # next sorting type
  }       # next report type
}


#
# table description
#
sub display_table_description() {

  my ($itemno, $section, $current, $last, $sortype, $reporttype) = @_;

  print "<hr><br>";

  print "<table align=center cellpadding=1 cellspacing=0 border=0 width=$width>\n";
  print "<b>$title section $section of $sections</b>\n";
  print "<tr><td>&nbsp;</td><tr bgcolor=$bgcolors[(++ $j) % 2]><td>&nbsp;</td>";
  print "<td colspan=15 align=center>Displaying ";
  print $itemno;
  print " items [";
  print $current + 1;
  print "-";
  print $last + 1;
  print "] of $total_items sorted by $sorttype $reporttype</td></tr>\n";
  print "<tr bgcolor=$bgcolors[(++ $j) % 2]>\n";
  print "</tr>\n";
  print "<tr><td>&nbsp;</td>\n";
  print "</table>\n";
}


#
# table header
#
sub display_table_header() {

  my ($itemno, $sorttype) = @_;

  my ($uno, $due, $tre);
  my ($sort1, $sort2, $sort4);

  if ($sorttype eq "ascending") {
    $sort1 = "/plugins/pep?$__progname__&sort=descending&field=hostname";
    $sort2 = "/plugins/pep?$__progname__&sort=descending&field=hwaddress";
    $sort4 = "/plugins/pep?$__progname__&sort=descending&field=ipaddress";
  }
  else {
    $sort1 = "/plugins/pep?$__progname__&sort=ascending&field=hostname";
    $sort2 = "/plugins/pep?$__progname__&sort=ascending&field=hwaddress";
    $sort4 = "/plugins/pep?$__progname__&sort=ascending&field=ipaddress";
  }

  if ($itemno == 1) {
    $uno = "Hostname";
    $due = "IP&nbsp;Address";
    $tre = "Hw&nbsp;Address";
  }
  else {
    $uno = "<a href=$sort1>Hostname</a>";
    $due = "<a href=$sort4>IP&nbsp;Address</a>";
    $tre = "<a href=$sort2>Hw&nbsp;Address</a>";
  }

  print "<table align=center cellpadding=1 cellspacing=0 border=0 width=$width>\n";

  # table header (column's name separated by blanks)
  print "<tr bgcolor=$headerbg>\n";
  print "<td width=10>&nbsp;</td>\n";
  print "<td align=left><font size=4><b>#</b></font></td>\n";     # itemno
  print "<td width=20>&nbsp;</td>\n";
  print "<td align=left><font size=4><b>$uno</b></font></td>\n";  # Hostname
  print "<td width=20>&nbsp;</td>\n";
  print "<td align=left><font size=4><b>$due</b></font></td>\n";  # IP Address
  print "<td width=20>&nbsp;</td>\n";
  print "<td align=left><font size=4><b>$tre</b></font></td>\n";  # Hw Address
  print "<td width=10>&nbsp;</td>\n";
  print "</tr>";
}


#
# the 'meat'
#
sub display_table() {

  my ($refa) = @_;

  my @sub_sorted = @$refa;

  print "<table align=center cellpadding=1 cellspacing=0 border=0 width=$width>\n";

  my $no = $current + 1;
  foreach $key (@sub_sorted) {

    # bgcolor - separator - value(s) - separator
    print "<tr bgcolor=$bgcolors[(++ $j) % 2]>\n";
    print "<td width=10>&nbsp;</td>\n";
    print "<td align=left>$no</td>\n";                      # itemno
    print "<td width=20>&nbsp;</td>\n";
    print "<td align=left>$key->{hostname}&nbsp;</td>\n";   # Hostname
    print "<td width=20>&nbsp;</td>\n";
    print "<td align=left>$key->{ipaddress}&nbsp;</td>\n";  # IP Address
    print "<td width=20>&nbsp;</td>\n";
    print "<td align=left>$key->{hwaddress}&nbsp;</td>\n";  # Hw Address
    print "<td width=10>&nbsp;</td>\n";
    print "</tr>\n";

    $no++;
  } # next item on the same section

  # end table
  print "</table>";
}

#
#
#
sub display_table_footer() {
  print "<br>";
}


#
# experimental
#

#
# Print a Table.
#
# $a is a reference to the array of the references to the hashes with the values
# $fields is a reference to the array of the keys to print
# $size is the size of the table in pixel
# $color is a reference to the hash with the specification of the colors and images to use
# $sorted is ...
#
sub table_print {

  my ($a, $fields, $size, $color, $title, $subtitle) = @_;

  if (! ref $a) {
    return;
  }

  print "<table bgcolor=$color->{colors}{bg} border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"$size\">";

  $cols = 1 + scalar @$fields / 3;

  #
  # this is the header of the table that is in turn another table
  # (1 row and 3 cols) with two small images at the left and right size
  # and a title and a subtitle in the middle (centered)
  #
  table_header($color, $cols, $title, $subtitle);

  my $i = 0;
  my $k = scalar @{$color->{rowcolors}};

  # the row for the description 
  table_row_description($color->{rowcolors}[1], $fields);

  # and now print a table with all the information
  foreach my $e (@$a) {

    # @rows is the array to the values to print in a row
    my @rows = ();

    my $kk = scalar @$fields;

    for ($jj = 0; $jj < $kk; $jj += 3) {

      my $info;
      #
      # special case to manage firstseen and lastseeen dates
      #
      if ($fields->[$jj + 0] eq "firstseen" || $fields->[$jj + 0] eq "lastseen") {
	$info = seen($e->{$fields->[$jj]});
      }
      else {
	$info = $e->{$fields->[$jj]};
      }

      my @specs = ($info, $fields->[$jj + 1], $fields->[$jj + 2]);

      @rows = (@rows, \@specs);
    }

    table_row($color->{rowcolors}[$i % $k], ++ $i, \@rows, $cols);
  }

  table_footer($cols, $color->{images}{bottom});

  print "</table>";
}


1;
