#
# unresolved.pl - it displays the table of all hosts currently unresolved.
#
# Copyright (C) 2003 Rocco Carbone <rocco@ntop.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

use lib qw(plugins/pep);
require "fmthtml.pl";
require "colorinfo.pl";
require "render.pl";
require "sorting.pl";
require "date.pl";

# useful variables
#
$__progname__   = "unresolved.pl";
$__version__    = "0.0.1";
$__updated__    = "2003 Mar 10";
$__boottime__   = time();
$__bootdate__   = localtime($__boottime__);

# Hey Netscape, this is for you
my $htmltitle   = "NTOP Statistics -- Unresolved Table";
my $bodyname    = "Traffic Report";
my $title       = "Traffic Summary";
my $description = "Shows the table of hosts currently unresolved";

#
# build various arrays from the PeP's %pephosts hash
#
my @vals = values %pephosts;

# this is the array of numerically sorted IP Address
my @sorted_by_ip = sort ipnumerically @vals;

# this is the sub-array that are not currently resolved
my @unresolved = ();
foreach my $s (@sorted_by_ip) {
  if ($s->{hostname} =~ /unresolved/) {
    @unresolved = (@unresolved, $s);
  }
}

my %unresolved = (
		  vals     => \@unresolved,
		  fields   => [
			       "hwaddress", '50%', "left",
			       "ipaddress", '40%', "left",
			       "pktsent",   '10%', "right",
			       "pktreceived",   '10%', "right",
			       "bytessent",   '10%', "right",
			      ],
		  color    => $colorinfo{red},
		  title    => "Top " . scalar @unresolved . " Hosts as known to ntop",
		  subtitle => "All the unresolved IP Addresses"
	    );

# what should be displayed
my @views = (\%unresolved);


#
# yet another main here
#
select STDOUT;

#
# print HTML header and title
#
html_header($htmltitle);

#
# start printing HTML body
#
html_start_body($bodyname);

#
# print report's title with its brief description
#
print "<center><font size=\"6\"><b>$description</b></font></center>\n";

#
# print the matrix
#
print_matrix(\@views, 2, 400);

#
# finish HTML output
#
html_stop_body();

1;
